/**
 * Copyright (c) 2017 Tomek Wójcik <tomek@bthlabs.pl>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

import ClassName from "classnames";
import React from "react";
import {Link} from "react-router";

import BaseView from "../lib/BaseView";
import ChartComponent from "../components/ChartComponent";
import ContainerComponent from "../components/ContainerComponent";
import LoaderComponent from "../components/LoaderComponent";
import WeaponIconComponent from "../components/WeaponIconComponent";

class NavComponent extends React.Component {
  render () {
    let previousClassName = ClassName("previous", {
      disabled: !this.props.previousDay
    });

    let nextClassName = ClassName("next", {
      disabled: !this.props.nextDay
    });

    let previousDayLinkTo = this.props.previousDay ? "/sessions/" + this.props.previousDay : "";
    let nextDayLinkTo = this.props.nextDay ? "/sessions/" + this.props.nextDay : "";

    return (
      <nav>
        <ul className="pager">
          <li className={previousClassName}>
            <Link to={previousDayLinkTo}>&larr; Previous</Link>
          </li>

          <li className={nextClassName}>
            <Link to={nextDayLinkTo}>Next &rarr;</Link>
          </li>
        </ul>
      </nav>
    );
  }
}

NavComponent.propTypes = {
  previousDay: React.PropTypes.string,
  nextDay: React.PropTypes.string
};

class GamesTableView extends React.Component {
  render () {
    return (
      <table className="table table-striped table-bordered table-hover">
        <thead>
          <tr>
            <th rowSpan="2">Map</th>
            <th rowSpan="2">Player</th>
            <th colSpan="4" className="text-center">Scores</th>
            <th rowSpan="2">Favourite weapon</th>
            <th rowSpan="2">Quad freak?</th>
          </tr>

          <tr>
            <th>Total</th>
            <th>Frags</th>
            <th>Deaths</th>
            <th>Suicides</th>
          </tr>
        </thead>

        <tbody>
          {(this.props.games.length == 0) &&
            <tr>
              <td colSpan="8"><strong>No stats :(</strong></td>
            </tr>
          }

          {this.props.games.map((game) => {
            return game.scores.map((score, index) => {
              let playerStatsLinkTo = "/players/" + score.player + "/game/" + game.uuid;

              return (
                <tr key={game.uuid + index}>
                  {(index == 0) &&
                    <td rowSpan={game.scores.length}>{game.map}</td>
                  }
                  <td>
                    <Link to={playerStatsLinkTo}>{score.player}</Link>
                  </td>
                  <td className="text-right">
                    <strong>{score.score}</strong>
                  </td>
                  <td className="text-right">{score.kills}</td>
                  <td className="text-right">{score.deaths}</td>
                  <td className="text-right">{score.suicides}</td>
                  <td className="text-right">
                    {(!score.favourite_weapon) && "?"}
                    {(score.favourite_weapon) &&
                      <WeaponIconComponent weapon={score.favourite_weapon} />
                    }
                  </td>
                  <td className="text-right">
                    {(!score.quad_freak) && "No"}
                    {(score.quad_freak) &&
                      "Yes (" + score.quad_pickups + " pickups)"
                    }
                  </td>
                </tr>
              );
            });
          })}
        </tbody>
      </table>
    );
  }
}

GamesTableView.propTypes = {
  games: React.PropTypes.array.isRequired
};

class SessionsView extends BaseView {
  constructor (props) {
    super(props);

    this.chartConfig = {
      "chart": {
        "type": "column"
      },
      "title": {
        "text": "Scores by map and player"
      },
      "credits": {
        "enabled": false
      },
      "yAxis": {
        "min": -10,
        "title": {
            "text": "Score"
        }
      },
      "plotOptions": {
        "column": {
            "pointPadding": 0.2,
            "borderWidth": 0
        }
      }
    };

    this.state = {
      shouldReload: false,
      ready: false,
      day: "",
      previousDay: "",
      nextDay: "",
      games: [],
      categories: null,
      series: []
    };
  }
  componentDidMount () {
    this.loadData();
  }
  componentDidUpdate (prevProps) {
    if ((this.state.shouldReload) && (!this.state.ready)) {
      this.setState({shouldReload: false});
      this.loadData();
    }
  }
  componentWillReceiveProps (nextProps) {
    if (nextProps.params.day != this.props.params.day) {
      this.setState({
        shouldReload: true,
        ready: false,
        day: "",
        previousDay: "",
        nextDay: "",
        games: [],
        categories: null,
        series: []
      });
    }
  }
  loadData () {
    this.fetch("/api/v1/sessions/" + (this.props.params.day || "")).then((data) => {
      this.setState({
        shouldReload: false,
        ready: true,
        day: data.day,
        previousDay: data.previous_day,
        nextDay: data.next_day,
        games: data.games
      });

      this.loadChartData();
    });
  }
  loadChartData () {
    this.fetch("/api/v1/charts/day/" + this.state.day).then((data) => {
      this.setState({
        categories: data.maps,
        series: data.scores
      });
    });
  }
  render () {
    return (
      <ContainerComponent>
        {this.state.ready &&
          <div>
            <NavComponent
              previousDay={this.state.previousDay}
              nextDay={this.state.nextDay} />

            <h2>Results</h2>

            <ChartComponent
              config={this.chartConfig}
              categories={this.state.categories}
              series={this.state.series}
              subtitle={this.state.day} />

            <h2>Stats</h2>

            <GamesTableView
              games={this.state.games} />

            <NavComponent
              onNavigateToDay={this.onNavigateToDay}
              previousDay={this.state.previousDay}
              nextDay={this.state.nextDay} />
          </div>
        }

        <LoaderComponent visible={!this.state.ready} />
      </ContainerComponent>
    );
  }
}

export default SessionsView;
